import pytest
import spacy
import copy
import negation
from termsets import termset
from spacy.pipeline import EntityRuler


def build_docs():
    docs = list()
    docs.append(
        (
            "She does not like Steve Jobs but likes Apple products.",
            [("Steve Jobs", True), ("Apple", False)],
        )
    )
    docs.append(
        (
            "Patient denies December 31, 2020 but has January 1, 2021. He likes USA.",
            [("December 31, 2020", True), ("January 1, 2021", False), ("USA", False)],
        )
    )
    docs.append(
        (
            "No history of USA, Germany, Italy, Canada, or Brazil",
            [
                ("USA", True),
                ("Germany", True),
                ("Italy", True),
                ("Canada", True),
                ("Brazil", True),
            ],
        )
    )

    docs.append(("That might not be Barack Obama.", [("Barack Obama", False)]))

    return docs


def test():
    nlp = spacy.load("en_core_web_sm")
    nlp.add_pipe("negex", last=True)
    docs = build_docs()
    for d in docs:
        doc = nlp(d[0])
        for i, e in enumerate(doc.ents):
            print(e.text, e._.negex)
            assert (e.text, e._.negex) == d[1][i]


def test_en():
    nlp = spacy.load("en_core_web_sm")
    ts = termset("en")
    nlp.add_pipe("negex", config={"neg_termset": ts.get_patterns()}, last=True)
    docs = build_docs()
    for d in docs:
        doc = nlp(d[0])
        for i, e in enumerate(doc.ents):
            print(e.text, e._.negex)
            assert (e.text, e._.negex) == d[1][i]


def test_own_terminology():
    nlp = spacy.load("en_core_web_sm")
    nlp.add_pipe(
        "negex",
        config={
            "neg_termset": {
                "pseudo_negations": [""],
                "preceding_negations": ["not"],
                "following_negations": [],
                "termination": ["whatever"],
            }
        },
        last=True,
    )
    doc = nlp("He does not like Steve Jobs whatever he says about Barack Obama.")
    assert doc.ents[1]._.negex == False


def test_issue7():
    nlp = spacy.load("en_core_web_sm")
    nlp.add_pipe("negex", last=True)
    ruler = EntityRuler(nlp)
    patterns = [{"label": "SOFTWARE", "pattern": "spacy"}]
    doc = nlp("fgfgdghgdh")


def test_get_patterns():
    # nlp = spacy.load("en_core_web_sm")
    # negex = Negex(nlp)
    # patterns = negex.get_patterns()
    ts = termset("en")
    patterns = ts.get_patterns()
    assert type(patterns) == dict
    assert len(patterns) == 4


def test_add_remove_patterns():
    ts = termset("en_clinical")
    patterns = copy.deepcopy(ts.get_patterns())
    ts.add_patterns(
        {
            "pseudo_negations": ["my favorite pattern"],
            "termination": ["these are", "great patterns", "but"],
            "preceding_negations": ["wow a negation"],
            "following_negations": ["extra negation"],
        }
    )
    patterns_after = ts.get_patterns()

    assert len(patterns_after["pseudo_negations"]) - 1 == len(
        patterns["pseudo_negations"]
    )
    assert len(patterns_after["termination"]) - 2 == len(patterns["termination"])
    assert len(patterns_after["preceding_negations"]) - 1 == len(
        patterns["preceding_negations"]
    )
    assert len(patterns_after["following_negations"]) - 1 == len(
        patterns["following_negations"]
    )

    ts.remove_patterns(
        {
            "termination": ["these are", "great patterns"],
            "pseudo_negations": ["my favorite pattern"],
            "preceding_negations": ["denied", "wow a negation"],
            "following_negations": ["unlikely", "extra negation"],
        }
    )
    patterns_after = ts.get_patterns()
    assert len(patterns_after["termination"]) == len(patterns["termination"])
    assert (
        len(patterns_after["following_negations"])
        == len(patterns["following_negations"]) - 1
    )
    assert (
        len(patterns_after["preceding_negations"])
        == len(patterns["preceding_negations"]) - 1
    )
    assert len(patterns_after["pseudo_negations"]) == len(patterns["pseudo_negations"])


if __name__ == "__main__":
    test()
    test_en()
    test_bad_beharor()
    test_own_terminology()
    test_get_patterns()
    test_issue7()
    test_add_remove_patterns()



import pytest
import spacy
import copy
import negation
from termsets import termset
from spacy.pipeline import EntityRuler


def build_med_docs():
    docs = list()
    docs.append(
        (
            "Patient denies cardiovascular disease but has headaches. No history of smoking. Alcoholism unlikely. Smoking not ruled out.",
            [
                ("Patient denies", False),
                ("cardiovascular disease", True),
                ("smoking", True),
                ("Alcoholism", True),
                ("unlikely", False),
                ("Smoking", False),
            ],
        )
    )
    docs.append(
        (
            "No history of headaches, prbc, smoking, acid reflux, or GERD.",
            [
                ("No history", True),
                ("headaches", True),
                ("prbc", True),
                ("smoking", True),
                ("acid reflux", True),
                ("GERD", True),
            ],
        )
    )

    docs.append(
        (
            "Alcoholism was not the cause of liver disease.",
            [("Alcoholism", True), ("liver disease", False)],
        )
    )

    return docs


def test_umls():
    nlp = spacy.load("en_core_sci_sm")
    ts = termset("en_clinical")
    nlp.add_pipe(
        "negex",
        config={
            "neg_termset": ts.get_patterns(),
            "ent_types": ["ENTITY"],
            "chunk_prefix": ["no"],
        },
        last=True,
    )
    # negex = Negex(
    #     nlp, language="en_clinical", ent_types=["ENTITY"], chunk_prefix=["no"]
    # )
    # nlp.add_pipe("negex", last=True)
    docs = build_med_docs()
    for d in docs:
        doc = nlp(d[0])
        print(doc.ents)
        for i, e in enumerate(doc.ents):
            print(e.text, e._.negex)
            assert (e.text, e._.negex) == d[1][i]


def __test_umls2():
    nlp = spacy.load("en_core_sci_sm")
    # negex = Negex(
    #     nlp, language="en_clinical_sensitive", ent_types=["ENTITY"], chunk_prefix=["no"]
    # )
    # nlp.add_pipe("negex", last=True)
    ts = termset("en_clinical")
    nlp.add_pipe(
        "negex",
        config={
            "neg_termset": ts.get_patterns(),
            "ent_types": ["ENTITY"],
            "chunk_prefix": ["no"],
        },
        last=True,
    )
    docs = build_med_docs()
    for d in docs:
        doc = nlp(d[0])
        for i, e in enumerate(doc.ents):
            print(e.text, e._.negex)
            assert (e.text, e._.negex) == d[1][i]


def test_issue_14():
    nlp = spacy.load("en_core_sci_sm")
    # negex = Negex(nlp, language="en_clinical", chunk_prefix=["no", "cancer free"])
    # negex.remove_patterns(following_negations="free")
    ts = termset("en_clinical")
    ts.remove_patterns({"following_negations": ["free"]})
    print(ts.get_patterns())
    nlp.add_pipe(
        "negex",
        config={
            "neg_termset": ts.get_patterns(),
            "chunk_prefix": ["no", "cancer free"],
        },
        last=True,
    )

    doc = nlp("The patient has a cancer free diagnosis")
    expected = [False, True]
    for i, e in enumerate(doc.ents):
        print(e.text, e._.negex)
        assert e._.negex == expected[i]

    nlp.remove_pipe("negex")
    # negex = Negex(nlp, language="en_clinical", chunk_prefix=["no", "free"])
    # nlp.add_pipe("negex", last=True)
    ts = termset("en_clinical")
    nlp.add_pipe(
        "negex",
        config={
            "neg_termset": ts.get_patterns(),
            "chunk_prefix": ["no", "free"],
        },
        last=True,
    )
    doc = nlp("The patient has a cancer free diagnosis")
    expected = [False, False]
    for i, e in enumerate(doc.ents):
        print(e.text, e._.negex)
        assert e._.negex == expected[i]


if __name__ == "__main__":
    test_umls()
    test_umls2()
    test_issue_14()



from spacy.language import Language
from spacy.tokens import Token, Doc, Span
from spacy.matcher import PhraseMatcher
import logging

from negspacy.termsets import termset

default_ts = termset("en_clinical").get_patterns()

# def create_negex_component(nlp: Language, name: str, termset_lang: str, ent_types: list, extension_name: str, pseudo_negations: list, preceding_negations: list, following_negations: list, termination: list, chunk_prefix: list):
#     return Negex(nlp, termset_lang, ent_types, extension_name, pseudo_negations, preceding_negations, following_negations, termination, chunk_prefix)
@Language.factory(
    "negex",
    default_config={
        "neg_termset": default_ts,
        "ent_types": list(),
        "extension_name": "negex",
        "chunk_prefix": list(),
    },
)
class Negex:
    """
        A spaCy pipeline component which identifies negated tokens in text.

        Based on: NegEx - A Simple Algorithm for Identifying Negated Findings and Diseasesin Discharge Summaries
    Chapman, Bridewell, Hanbury, Cooper, Buchanan

    Parameters
    ----------
    nlp: object
        spaCy language object
    ent_types: list
        list of entity types to negate
    termset_lang: str
        language code, if using default termsets (e.g. "en" for english)
    extension_name: str
        defaults to "negex"; whether entity is negated is then available as ent._.negex
    pseudo_negations: list
        list of phrases that cancel out a negation, if empty, defaults are used
    preceding_negations: list
        negations that appear before an entity, if empty, defaults are used
    following_negations: list
        negations that appear after an entity, if empty, defaults are used
    termination: list
        phrases that "terminate" a sentence for processing purposes such as "but". If empty, defaults are used

    """

    def __init__(
        self,
        nlp: Language,
        name: str,
        neg_termset: dict,
        ent_types: list,
        extension_name: str,
        chunk_prefix: list,
    ):
        # if not termset_lang in LANGUAGES:
        #     raise KeyError(
        #         f"{termset_lang} not found in languages termset. "
        #         "Ensure this is a supported termset or specify "
        #         "your own termsets when initializing Negex."
        #     )
        # termsets = LANGUAGES[termset_lang]
        if not Span.has_extension(extension_name):
            Span.set_extension(extension_name, default=False, force=True)

        ts = neg_termset
        expected_keys = [
            "pseudo_negations",
            "preceding_negations",
            "following_negations",
            "termination",
        ]
        if not set(ts.keys()) == set(expected_keys):
            raise KeyError(
                f"Unexpected or missing keys in 'neg_termset', expected: {expected_keys}, instead got: {list(ts.keys())}"
            )

        self.pseudo_negations = ts["pseudo_negations"]
        self.preceding_negations = ts["preceding_negations"]
        self.following_negations = ts["following_negations"]
        self.termination = ts["termination"]

        self.nlp = nlp
        self.ent_types = ent_types
        self.extension_name = extension_name
        self.build_patterns()
        self.chunk_prefix = list(nlp.tokenizer.pipe(chunk_prefix))

    def build_patterns(self):
        # efficiently build spaCy matcher patterns
        self.matcher = PhraseMatcher(self.nlp.vocab, attr="LOWER")

        self.pseudo_patterns = list(self.nlp.tokenizer.pipe(self.pseudo_negations))
        self.matcher.add("pseudo", None, *self.pseudo_patterns)

        self.preceding_patterns = list(
            self.nlp.tokenizer.pipe(self.preceding_negations)
        )
        self.matcher.add("Preceding", None, *self.preceding_patterns)

        self.following_patterns = list(
            self.nlp.tokenizer.pipe(self.following_negations)
        )
        self.matcher.add("Following", None, *self.following_patterns)

        self.termination_patterns = list(self.nlp.tokenizer.pipe(self.termination))
        self.matcher.add("Termination", None, *self.termination_patterns)

    # def remove_patterns(
    #     self,
    #     pseudo_negations=None,
    #     preceding_negations=None,
    #     following_negations=None,
    #     termination=None,
    # ):
    #     if pseudo_negations:
    #         if isinstance(pseudo_negations, list):
    #             for p in pseudo_negations:
    #                 self.pseudo_negations.remove(p)
    #         else:
    #             self.pseudo_negations.remove(pseudo_negations)
    #     if preceding_negations:
    #         if isinstance(preceding_negations, list):
    #             for p in preceding_negations:
    #                 self.preceding_negations.remove(p)
    #         else:
    #             self.preceding_negations.remove(preceding_negations)
    #     if following_negations:
    #         if isinstance(following_negations, list):
    #             for p in following_negations:
    #                 self.following_negations.remove(p)
    #         else:
    #             self.following_negations.remove(following_negations)
    #     if termination:
    #         if isinstance(termination, list):
    #             for p in termination:
    #                 self.termination.remove(p)
    #         else:
    #             self.termination.remove(termination)
    #     self.build_patterns()

    # def add_patterns(
    #     self,
    #     pseudo_negations=None,
    #     preceding_negations=None,
    #     following_negations=None,
    #     termination=None,
    # ):
    #     if pseudo_negations:
    #         if not isinstance(pseudo_negations, list):
    #             raise ValueError("A list of phrases expected when adding patterns")
    #         self.pseudo_negations.extend(pseudo_negations)
    #     if preceding_negations:
    #         if not isinstance(preceding_negations, list):
    #             raise ValueError("A list of phrases expected when adding patterns")
    #         self.preceding_negations.extend(preceding_negations)
    #     if following_negations:
    #         if not isinstance(following_negations, list):
    #             raise ValueError("A list of phrases expected when adding patterns")
    #         self.following_negations.extend(following_negations)
    #     if termination:
    #         if not isinstance(termination, list):
    #             raise ValueError("A list of phrases expected when adding patterns")
    #         self.termination.extend(termination)
    #     self.build_patterns()

    # def get_patterns(self):
    #     """
    #     returns phrase patterns used for various negation dictionaries

    #     Returns
    #     -------
    #     patterns: dict
    #         pattern_type: [patterns]

    #     """
    #     patterns = {
    #         "pseudo_patterns": self.pseudo_patterns,
    #         "preceding_patterns": self.preceding_patterns,
    #         "following_patterns": self.following_patterns,
    #         "termination_patterns": self.termination_patterns,
    #     }
    #     for pattern in patterns:
    #         logging.info(pattern)
    #     return patterns

    def process_negations(self, doc):
        """
        Find negations in doc and clean candidate negations to remove pseudo negations

        Parameters
        ----------
        doc: object
            spaCy Doc object

        Returns
        -------
        preceding: list
            list of tuples for preceding negations
        following: list
            list of tuples for following negations
        terminating: list
            list of tuples of terminating phrases

        """
        ###
        # does not work properly in spacy 2.1.8. Will incorporate after 2.2.
        # Relying on user to use NER in meantime
        # see https://github.com/jenojp/negspacy/issues/7
        ###
        # if not doc.is_nered:
        #     raise ValueError(
        #         "Negations are evaluated for Named Entities found in text. "
        #         "Your SpaCy pipeline does not included Named Entity resolution. "
        #         "Please ensure it is enabled or choose a different language model that includes it."
        #     )
        preceding = list()
        following = list()
        terminating = list()

        matches = self.matcher(doc)
        pseudo = [
            (match_id, start, end)
            for match_id, start, end in matches
            if self.nlp.vocab.strings[match_id] == "pseudo"
        ]

        for match_id, start, end in matches:
            if self.nlp.vocab.strings[match_id] == "pseudo":
                continue
            pseudo_flag = False
            for p in pseudo:
                if start >= p[1] and start <= p[2]:
                    pseudo_flag = True
                    continue
            if not pseudo_flag:
                if self.nlp.vocab.strings[match_id] == "Preceding":
                    preceding.append((match_id, start, end))
                elif self.nlp.vocab.strings[match_id] == "Following":
                    following.append((match_id, start, end))
                elif self.nlp.vocab.strings[match_id] == "Termination":
                    terminating.append((match_id, start, end))
                else:
                    logging.warnings(
                        f"phrase {doc[start:end].text} not in one of the expected matcher types."
                    )
        return preceding, following, terminating

    def termination_boundaries(self, doc, terminating):
        """
        Create sub sentences based on terminations found in text.

        Parameters
        ----------
        doc: object
            spaCy Doc object
        terminating: list
            list of tuples with (match_id, start, end)

        returns
        -------
        boundaries: list
            list of tuples with (start, end) of spans

        """
        sent_starts = [sent.start for sent in doc.sents]
        terminating_starts = [t[1] for t in terminating]
        starts = sent_starts + terminating_starts + [len(doc)]
        starts.sort()
        boundaries = list()
        index = 0
        for i, start in enumerate(starts):
            if not i == 0:
                boundaries.append((index, start))
            index = start
        return boundaries

    def negex(self, doc):
        """
        Negates entities of interest

        Parameters
        ----------
        doc: object
            spaCy Doc object

        """
        preceding, following, terminating = self.process_negations(doc)
        boundaries = self.termination_boundaries(doc, terminating)
        for b in boundaries:
            sub_preceding = [i for i in preceding if b[0] <= i[1] < b[1]]
            sub_following = [i for i in following if b[0] <= i[1] < b[1]]

            for e in doc[b[0] : b[1]].ents:
                if self.ent_types:
                    if e.label_ not in self.ent_types:
                        continue
                if any(pre < e.start for pre in [i[1] for i in sub_preceding]):
                    e._.set(self.extension_name, True)
                    continue
                if any(fol > e.end for fol in [i[2] for i in sub_following]):
                    e._.set(self.extension_name, True)
                    continue
                if self.chunk_prefix:
                    if any(
                        e.text.lower().startswith(c.text.lower())
                        for c in self.chunk_prefix
                    ):
                        e._.set(self.extension_name, True)
        return doc

    def __call__(self, doc):
        return self.negex(doc)



"""
Default termsets for various languages
"""

LANGUAGES = dict()

# english termset dictionary
en = dict()
pseudo = [
    "no further",
    "not able to be",
    "not certain if",
    "not certain whether",
    "not necessarily",
    "without any further",
    "without difficulty",
    "without further",
    "might not",
    "not only",
    "no increase",
    "no significant change",
    "no change",
    "no definite change",
    "not extend",
    "not cause",
]
en["pseudo_negations"] = pseudo

preceding = [
    "absence of",
    "declined",
    "denied",
    "denies",
    "denying",
    "no sign of",
    "no signs of",
    "not",
    "not demonstrate",
    "symptoms atypical",
    "doubt",
    "negative for",
    "no",
    "versus",
    "without",
    "doesn't",
    "doesnt",
    "don't",
    "dont",
    "didn't",
    "didnt",
    "wasn't",
    "wasnt",
    "weren't",
    "werent",
    "isn't",
    "isnt",
    "aren't",
    "arent",
    "cannot",
    "can't",
    "cant",
    "couldn't",
    "couldnt",
    "never",
]
en["preceding_negations"] = preceding

following = [
    "declined",
    "unlikely",
    "was not",
    "were not",
    "wasn't",
    "wasnt",
    "weren't",
    "werent",
]
en["following_negations"] = following

termination = [
    "although",
    "apart from",
    "as there are",
    "aside from",
    "but",
    "except",
    "however",
    "involving",
    "nevertheless",
    "still",
    "though",
    "which",
    "yet",
]
en["termination"] = termination

LANGUAGES["en"] = en

# en_clinical builds upon en
en_clinical = dict()
pseudo_clinical = pseudo + [
    "gram negative",
    "not rule out",
    "not ruled out",
    "not been ruled out",
    "not drain",
    "no suspicious change",
    "no interval change",
    "no significant interval change",
]
en_clinical["pseudo_negations"] = pseudo_clinical

preceding_clinical = preceding + [
    "patient was not",
    "without indication of",
    "without sign of",
    "without signs of",
    "without any reactions or signs of",
    "no complaints of",
    "no evidence of",
    "no cause of",
    "evaluate for",
    "fails to reveal",
    "free of",
    "never developed",
    "never had",
    "did not exhibit",
    "rules out",
    "rule out",
    "rule him out",
    "rule her out",
    "rule patient out",
    "rule the patient out",
    "ruled out",
    "ruled him out",
    "ruled her out",
    "ruled patient out",
    "ruled the patient out",
    "r/o",
    "ro",
]
en_clinical["preceding_negations"] = preceding_clinical

following_clinical = following + ["was ruled out", "were ruled out", "free"]
en_clinical["following_negations"] = following_clinical

termination_clinical = termination + [
    "cause for",
    "cause of",
    "causes for",
    "causes of",
    "etiology for",
    "etiology of",
    "origin for",
    "origin of",
    "origins for",
    "origins of",
    "other possibilities of",
    "reason for",
    "reason of",
    "reasons for",
    "reasons of",
    "secondary to",
    "source for",
    "source of",
    "sources for",
    "sources of",
    "trigger event for",
]
en_clinical["termination"] = termination_clinical
LANGUAGES["en_clinical"] = en_clinical

en_clinical_sensitive = dict()

preceding_clinical_sensitive = preceding_clinical + [
    "concern for",
    "supposed",
    "which causes",
    "leads to",
    "h/o",
    "history of",
    "instead of",
    "if you experience",
    "if you get",
    "teaching the patient",
    "taught the patient",
    "teach the patient",
    "educated the patient",
    "educate the patient",
    "educating the patient",
    "monitored for",
    "monitor for",
    "test for",
    "tested for",
]
en_clinical_sensitive["pseudo_negations"] = pseudo_clinical
en_clinical_sensitive["preceding_negations"] = preceding_clinical_sensitive
en_clinical_sensitive["following_negations"] = following_clinical
en_clinical_sensitive["termination"] = termination_clinical

LANGUAGES["en_clinical_sensitive"] = en_clinical_sensitive


# es_clinical
es_clinical = {
    "pseudo_negations": [
        "sin aumento",
        "ningún cambio",
        "sin cambios sospechosos",
        "ningún cambio significativo",
        "sin cambio de intervalo",
        "sin cambio definitivo",
        "no se extiende",
        "no causa",
        "no drena",
        "cambio de intervalo no significativo",
        "no estoy seguro si",
        "no estoy seguro de si",
        "gram negativo",
        "sin dificultad",
        "no necesariamente",
        "no solo",
        "duda",
        "tengo dudas",
        "dudo",
    ],
    "preceding_negations": [
        "ausencia de",
        "no pueden ver",
        "no poder",
        "revisado para",
        "rechazado",
        "declina",
        "negado",
        "niega",
        "negando",
        "evaluar por",
        "no revela",
        "libre de",
        "negativo para",
        "nunca desarrollado",
        "nunca tuve",
        "no",
        "no anormal",
        "ninguna causa de",
        "sin quejas de",
        "sin evidencia",
        "ninguna nueva evidencia",
        "ninguna otra evidencia",
        "ninguna evidencia para sugerir",
        "sin hallazgos de",
        "no hay hallazgos para indicar",
        "no hay evidencia mamográfica de",
        "nada nuevo",
        "ninguna evidencia radiográfica de",
        "ninguna señal de",
        "no significativo",
        "sin signos de",
        "ninguna sugerencia de",
        "no sospechoso",
        "no",
        "no aparece",
        "no apreciar",
        "no asociado con",
        "no me quejo de",
        "no demostrar",
        "no exhibir",
        "no sentir",
        "no tenía",
        "no tengo",
        "no saber de",
        "no se sabe que tiene",
        "no revelar",
        "no ver",
        "no ser",
        "paciente no era",
        "más bien que",
        "resuelto",
        "hacer una prueba por",
        "excluir",
        "nada especial para",
        "con ningún",
        "sin ninguna evidencia de",
        "sin evidencia",
        "sin indicación de",
        "sin signo de",
        "sin",
        "descartar para",
        "descartarlo por",
        "descartarla por",
        "descartar al paciente por",
        "descartarlo",
        "descartarla",
        "descartar",
        "r / o",
        "ro",
        "descartar al paciente",
        "excluye",
        "lo descarta",
        "la excluye",
        "expulsó al paciente por",
        "gobierna al paciente",
        "lo descartó contra",
        "la descartó contra",
        "lo descartó",
        "la descartó",
        "descartado contra",
        "descartó al paciente contra",
        "descartaron para",
        "descartaron contra",
        "descartó",
        "lo descartaron por",
        "lo descartaron en contra",
        "lo descartaron",
        "la descartaron por",
        "lo descartó contra",
        "lo descartó",
        "descartaron al paciente contra",
        "descartaron al paciente por",
        "descartó al paciente",
        "puede descartar",
        "puede descartar contra",
        "puede descartar",
        "puede descartarlo por",
        "puede descartarlo en contra",
        "puede descartarlo",
        "puede descartarla por",
        "puede descartarla contra",
        "puede descartarla",
        "puede descartar al paciente por",
        "puede descartar al paciente contra",
        "puede descartar al paciente",
        "adecuado para descartar",
        "adecuado para descartar",
        "adecuado para descartarlo por",
        "adecuado para descartarlo",
        "adecuado para descartarla por",
        "adecuado para descartarla",
        "adecuado para descartar al paciente por",
        "adecuado para descartar al paciente contra",
        "adecuado para descartar al paciente",
        "suficiente para descartar",
        "suficiente para descartar",
        "suficiente para descartar",
        "suficiente para descartarlo por",
        "suficiente para descartarlo en contra",
        "suficiente para descartarlo",
        "suficiente para descartarla por",
        "suficiente para descartarla en contra",
        "suficiente para descartarla",
        "suficiente para descartar al paciente por",
        "suficiente para descartar al paciente contra",
        "suficiente para descartar al paciente",
        "lo que debe descartarse es",
    ],
    "following_negations": [
        "debe descartarse para",
        "debe ser descartado para",
        "puede ser descartado para",
        "puede ser descartado para",
        "podría ser descartado por",
        "será descartado por",
        "se puede descartar por",
        "debe descartarse para",
        "debe ser descartado por",
        "ser descartado por",
        "improbable",
        "libre",
        "fue descartado",
        "está descartado",
        "están descartadas",
        "han sido descartadas",
        "ha sido descartado",
        "siendo descartado",
        "debe descartarse",
        "debe ser descartado",
        "puede ser descartado",
        "podría descartarse",
        "podría ser descartado",
        "será descartado",
        "se puede descartar",
        "debe descartarse",
        "debe ser descartado",
        "ser descartado",
        "se descarta",
    ],
    "termination": [
        "pero",
        "sin embargo",
        "sin embargo",
        "todavía",
        "aunque",
        "a pesar de que",
        "todavía",
        "aparte de",
        "excepto",
        "aparte de",
        "secundario a",
        "como la causa de",
        "como fuente de",
        "como la razón de",
        "como la etiología de",
        "como el origen de",
        "como la causa de",
        "como fuente de",
        "como la razón de",
        "como la etiología de",
        "como el origen de",
        "como la causa secundaria de",
        "como la fuente secundaria de",
        "como la razón secundaria de",
        "como la etiología secundaria de",
        "como el origen secundario de",
        "como la causa secundaria de",
        "como la fuente secundaria para",
        "como la razón secundaria para",
        "como la etiología secundaria para",
        "como el origen secundario para",
        "como causa de",
        "como fuente de",
        "como una razón de",
        "como una etiología de",
        "como causa de",
        "como fuente de",
        "como una razón para",
        "como una etiología para",
        "como una causa secundaria de",
        "como una fuente secundaria de",
        "como una razón secundaria de",
        "como una etiología secundaria de",
        "como un origen secundario de",
        "como una causa secundaria para",
        "como una fuente secundaria para",
        "como una razón secundaria para",
        "como una etiología secundaria para",
        "como un origen secundario para",
        "causa de",
        "motivo de",
        "causas de",
        "causas de",
        "fuente de",
        "fuente para",
        "fuentes de",
        "fuentes para",
        "razón de",
        "razón para",
        "razones de",
        "razones para",
        "etiología de",
        "etiología para",
        "desencadenar evento para",
        "origen de",
        "origen para",
        "orígenes de",
        "orígenes para",
        "otras posibilidades de",
    ],
}

LANGUAGES["es_clinical"] = es_clinical


class termset:
    def __init__(self, termset_lang):
        self.pattern_types = [
            "pseudo_negations",
            "preceding_negations",
            "following_negations",
            "termination",
        ]
        self.terms = LANGUAGES[termset_lang]

    def get_patterns(self):
        return self.terms

    def remove_patterns(self, pattern_dict):
        for key, value in pattern_dict.items():
            if key in self.pattern_types:
                self.terms[key] = [i for i in self.terms[key] if i not in value]
            else:
                raise ValueError(f"Unexpected key: {key} not in {self.pattern_types}")

    def add_patterns(self, pattern_dict):
        for key, value in pattern_dict.items():
            if key in self.pattern_types:
                self.terms[key] = list(set(self.terms[key] + value))
            else:
                raise ValueError(f"Unexpected key: {key} not in {self.pattern_types}")
